@@ -2,6 +2,8 @@ package DateTime;
 
 use strict;
 
+$DateTime::IsPurePerl = 1;
+
 my @MonthLengths =
     ( 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 );
 
@@ -139,8 +141,8 @@ sub _ymd2rd
     # add: day of month, days of previous 0-11 month period that began
     # w/March, days of previous 0-399 year period that began w/March
     # of a 400-multiple year), days of any 400-year periods before
-    # that, and 306 days to adjust from Mar 1, year 0-relative to Jan
-    # 1, year 1-relative (whew)
+    # that, and finally subtract 306 days to adjust from Mar 1, year
+    # 0-relative to Jan 1, year 1-relative (whew)
 
     $d += ( $m * 367 - 1094 ) / 12 + $y % 100 * 1461 / 4 +
           ( $y / 100 * 36524 + $y / 400 ) - 306;
@@ -199,20 +201,15 @@ sub _is_leap_year
     shift;
     my $year = shift;
 
-    if ($year % 400 == 0)
-    {
-        return 1;
-    }
-    elsif ($year % 100 == 0)
-    {
-        return 0;
-    }
-    elsif ($year % 4 == 0)
-    {
-        return 1;
-    }
+    # According to Bjorn Tackmann, this line prevents an infinite loop
+    # when running the tests under Qemu. I cannot reproduce this on
+    # Ubuntu or with Strawberry Perl on Win2K.
+    return 0 if $year == INFINITY() || $year == NEG_INFINITY();
+    return 0 if $year % 4;
+    return 1 if $year % 100;
+    return 0 if $year % 400;
 
-    return 0;
+    return 1;
 }
 
 sub _day_length { DateTime::LeapSecond::day_length($_[1]) }
